@@ -1,5 +1,5 @@
-#ifndef __AUTH_H
-#define __AUTH_H
+#ifndef AUTH_H
+#define AUTH_H
 
 #define PASSWORD_HIDDEN_STR "<hidden>"
 
@@ -11,9 +11,7 @@ struct auth_passdb {
 	unsigned int id;
 	const char *args;
 	struct passdb_module *passdb;
-#ifdef HAVE_MODULES
-	struct auth_module *module;
-#endif
+
         /* if user is found from this passdb, deny authentication immediately */
 	unsigned int deny:1;
 	/* after a successful lookup, continue to next passdb */
@@ -27,9 +25,6 @@ struct auth_userdb {
 	unsigned int num;
 	const char *args;
 	struct userdb_module *userdb;
-#ifdef HAVE_MODULES
-	struct auth_module *module;
-#endif
 };
 
 struct auth {
@@ -42,7 +37,7 @@ struct auth {
 	struct auth_passdb *passdbs;
 	struct auth_userdb *userdbs;
 
-	const char *const *auth_realms;
+	char *const *auth_realms;
 	const char *default_realm;
 	const char *anonymous_username;
 	const char *username_format;
@@ -51,7 +46,7 @@ struct auth {
 	char username_translation[256];
 	char master_user_separator;
 	bool ssl_require_client_cert;
-        bool ssl_username_from_cert;
+	bool ssl_username_from_cert;
 
 	bool verbose, verbose_debug, verbose_debug_passwords;
 };
