@@ -1,5 +1,5 @@
-#ifndef __USERDB_H
-#define __USERDB_H
+#ifndef USERDB_H
+#define USERDB_H
 
 #include "auth-stream.h"
 
@@ -13,7 +13,6 @@ enum userdb_result {
 };
 
 typedef void userdb_callback_t(enum userdb_result result,
-			       struct auth_stream_reply *reply,
 			       struct auth_request *request);
 
 struct userdb_module {
@@ -46,6 +45,12 @@ void userdb_preinit(struct auth *auth, const char *driver, const char *args);
 void userdb_init(struct auth_userdb *userdb);
 void userdb_deinit(struct auth_userdb *userdb);
 
+void userdb_register_module(struct userdb_module_interface *iface);
+void userdb_unregister_module(struct userdb_module_interface *iface);
+
+void userdbs_init(void);
+void userdbs_deinit(void);
+
 #include "auth-request.h"
 
 #endif
