@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2009 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2002-2010 Dovecot authors, see the included COPYING file */
 
 #include "common.h"
 #include "base64.h"
@@ -194,8 +194,10 @@ static bool client_handle_args(struct imap_client *client,
 		if (!success)
 			return FALSE;
 		if (imap_proxy_new(client, host, port, destuser, master_user,
-				   pass, ssl_flags, proxy_timeout_msecs) < 0)
-			client_auth_failed(client, TRUE);
+				   pass, ssl_flags, proxy_timeout_msecs) < 0) {
+			if (!client->destroyed)
+				client_auth_failed(client, TRUE);
+		}
 		return TRUE;
 	}
 
