@@ -19,6 +19,7 @@ enum index_cache_field {
 	MAIL_CACHE_IMAP_BODYSTRUCTURE,
 	MAIL_CACHE_IMAP_ENVELOPE,
 	MAIL_CACHE_POP3_UIDL,
+	MAIL_CACHE_GUID,
 	MAIL_CACHE_MESSAGE_PARTS,
 
 	MAIL_INDEX_CACHE_FIELD_COUNT
@@ -68,7 +69,6 @@ struct index_mail_line {
 struct message_header_line;
 
 struct index_mail_data {
-	enum mail_flags flags;
 	time_t date, received_date, save_date;
 	uoff_t virtual_size, physical_size;
 
@@ -82,12 +82,14 @@ struct index_mail_data {
 
 	uint32_t seq;
 	uint32_t cache_flags;
+	uint64_t modseq;
 	enum index_mail_access_part access_part;
 	/* dont_cache_fields overrides cache_fields */
 	enum mail_fetch_field cache_fetch_fields, dont_cache_fetch_fields;
 	unsigned int dont_cache_field_idx;
 
 	struct istream *stream, *filter_stream;
+	struct tee_istream *tee_stream;
 	struct message_size hdr_size, body_size;
 	struct message_parser_ctx *parser_ctx;
 	int parsing_count;
@@ -107,6 +109,7 @@ struct index_mail_data {
 	unsigned int header_parsed:1;
 	unsigned int no_caching:1;
 	unsigned int destroying_stream:1;
+	unsigned int initialized_wrapper_stream:1;
 };
 
 struct index_mail {
@@ -169,8 +172,10 @@ int index_mail_get_headers(struct mail *_mail, const char *field,
 int index_mail_get_header_stream(struct mail *_mail,
 				 struct mailbox_header_lookup_ctx *headers,
 				 struct istream **stream_r);
+void index_mail_set_read_buffer_size(struct mail *mail, struct istream *input);
 
 enum mail_flags index_mail_get_flags(struct mail *_mail);
+uint64_t index_mail_get_modseq(struct mail *_mail);
 const char *const *index_mail_get_keywords(struct mail *_mail);
 const ARRAY_TYPE(keyword_indexes) *
 index_mail_get_keyword_indexes(struct mail *_mail);
@@ -213,4 +218,7 @@ void index_mail_cache_parse_continue(struct mail *mail);
 void index_mail_cache_parse_deinit(struct mail *mail, time_t received_date,
 				   bool success);
 
+int index_mail_cache_lookup_field(struct index_mail *mail, buffer_t *buf,
+				  unsigned int field_idx);
+
 #endif
