@@ -1,4 +1,4 @@
-/* Copyright (c) 2003-2009 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2003-2010 Dovecot authors, see the included COPYING file */
 
 /*
    Version 1 format has been used for most versions of Dovecot up to v1.0.x.
@@ -123,6 +123,7 @@ struct maildir_uidlist_iter_ctx {
 	uint32_t prev_uid;
 };
 
+static int maildir_uidlist_open_latest(struct maildir_uidlist *uidlist);
 static bool maildir_uidlist_iter_next_rec(struct maildir_uidlist_iter_ctx *ctx,
 					  struct maildir_uidlist_rec **rec_r);
 
@@ -310,6 +311,7 @@ static void maildir_uidlist_reset(struct maildir_uidlist *uidlist)
 	maildir_uidlist_close(uidlist);
 	uidlist->last_seen_uid = 0;
 	uidlist->initial_hdr_read = FALSE;
+	uidlist->read_records_count = 0;
 
 	hash_table_clear(uidlist->files, FALSE);
 	array_clear(&uidlist->records);
@@ -376,15 +378,11 @@ static void maildir_uidlist_update_hdr(struct maildir_uidlist *uidlist,
 	}
 
 	mhdr = &uidlist->mbox->maildir_hdr;
-	if (mhdr->uidlist_mtime == 0) {
-		if (!uidlist->initial_read)
-			(void)maildir_uidlist_refresh(uidlist);
-		if (uidlist->version != UIDLIST_VERSION) {
-			/* upgrading from older verson. don't update the
-			   uidlist times until it uses the new format */
-			uidlist->recreate = TRUE;
-			return;
-		}
+	if (mhdr->uidlist_mtime == 0 && uidlist->version != UIDLIST_VERSION) {
+		/* upgrading from older verson. don't update the
+		   uidlist times until it uses the new format */
+		uidlist->recreate = TRUE;
+		return;
 	}
 	mhdr->uidlist_mtime = st->st_mtime;
 	mhdr->uidlist_mtime_nsecs = ST_MTIME_NSEC(*st);
@@ -399,10 +397,18 @@ maildir_uidlist_records_array_delete(struct maildir_uidlist *uidlist,
 	unsigned int idx, count;
 
 	recs = array_get(&uidlist->records, &count);
-	pos = bsearch(&rec, recs, count, sizeof(*recs), maildir_uid_cmp);
-	i_assert(pos != NULL);
-
-	idx = pos - recs;
+	if (!uidlist->unsorted) {
+		pos = bsearch(&rec, recs, count, sizeof(*recs),
+			      maildir_uid_cmp);
+		i_assert(pos != NULL);
+		idx = pos - recs;
+	} else {
+		for (idx = 0; idx < count; idx++) {
+			if (recs[idx]->uid == rec->uid)
+				break;
+		}
+		i_assert(idx != count);
+	}
 	array_delete(&uidlist->records, idx, 1);
 	return idx;
 }
@@ -731,7 +737,6 @@ maildir_uidlist_update_read(struct maildir_uidlist *uidlist,
 	if (ret > 0) {
 		uidlist->prev_read_uid = 0;
 		uidlist->change_counter++;
-		uidlist->read_records_count = 0;
 		uidlist->retry_rewind = last_read_offset != 0 && try_retry;
 
 		ret = 1;
@@ -867,11 +872,10 @@ maildir_uidlist_has_changed(struct maildir_uidlist *uidlist, bool *recreated_r)
 	}
 }
 
-int maildir_uidlist_refresh(struct maildir_uidlist *uidlist)
+static int maildir_uidlist_open_latest(struct maildir_uidlist *uidlist)
 {
-        unsigned int i;
-        bool retry, recreated;
-        int ret;
+	bool recreated;
+	int ret;
 
 	if (uidlist->fd != -1) {
 		ret = maildir_uidlist_has_changed(uidlist, &recreated);
@@ -881,10 +885,29 @@ int maildir_uidlist_refresh(struct maildir_uidlist *uidlist)
 			return ret < 0 ? -1 : 1;
 		}
 
-		if (recreated)
-			maildir_uidlist_close(uidlist);
+		if (!recreated)
+			return 0;
+		maildir_uidlist_close(uidlist);
 	}
 
+	uidlist->fd = nfs_safe_open(uidlist->path, O_RDWR);
+	if (uidlist->fd == -1 && errno != ENOENT) {
+		mail_storage_set_critical(uidlist->ibox->box.storage,
+			"open(%s) failed: %m", uidlist->path);
+		return -1;
+	}
+	return 0;
+}
+
+int maildir_uidlist_refresh(struct maildir_uidlist *uidlist)
+{
+        unsigned int i;
+        bool retry;
+        int ret;
+
+	if (maildir_uidlist_open_latest(uidlist) < 0)
+		return -1;
+
         for (i = 0; ; i++) {
 		ret = maildir_uidlist_update_read(uidlist, &retry,
 						i < UIDLIST_ESTALE_RETRY_COUNT);
@@ -1240,6 +1263,7 @@ maildir_uidlist_records_drop_expunges(struct maildir_uidlist *uidlist)
 		rec = mail_index_lookup(view, seq);
 		if (recs[i]->uid < rec->uid) {
 			/* expunged entry */
+			hash_table_remove(uidlist->files, recs[i]->filename);
 			i++;
 		} else if (recs[i]->uid > rec->uid) {
 			/* index isn't up to date. we're probably just
@@ -1252,8 +1276,10 @@ maildir_uidlist_records_drop_expunges(struct maildir_uidlist *uidlist)
 	}
 
 	/* drop messages expunged at the end of index */
-	while (i < count && recs[i]->uid < hdr->next_uid)
+	while (i < count && recs[i]->uid < hdr->next_uid) {
+		hash_table_remove(uidlist->files, recs[i]->filename);
 		i++;
+	}
 	/* view might not be completely up-to-date, so preserve any
 	   messages left */
 	for (; i < count; i++)
@@ -1423,18 +1449,12 @@ static int maildir_uidlist_sync_update(struct maildir_uidlist_sync_ctx *ctx)
 	if (maildir_uidlist_want_recreate(ctx))
 		return maildir_uidlist_recreate(uidlist);
 
-	if (uidlist->fd == -1) {
-		/* NOREFRESH flag used. we're just appending some messages. */
+	if (!uidlist->locked_refresh || uidlist->fd == -1) {
+		/* make sure we have the latest file (e.g. NOREFRESH used) */
 		i_assert(uidlist->initial_hdr_read);
-
-		uidlist->fd = nfs_safe_open(uidlist->path, O_RDWR);
-		if (uidlist->fd == -1) {
-			mail_storage_set_critical(storage,
-				"open(%s) failed: %m", uidlist->path);
+		if (maildir_uidlist_open_latest(uidlist) < 0)
 			return -1;
-		}
 	}
-
 	i_assert(ctx->first_unwritten_pos != (unsigned int)-1);
 
 	if (lseek(uidlist->fd, 0, SEEK_END) < 0) {
