@@ -1,4 +1,4 @@
-/* Copyright (c) 2006-2009 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2006-2010 Dovecot authors, see the included COPYING file */
 
 #include "lib.h"
 #include "hostpid.h"
@@ -76,8 +76,8 @@ fs_list_is_valid_common_nonfs(struct mailbox_list *list, const char *name)
 			if (maildir_len > 0 &&
 			    strncmp(p, list->set.maildir_name,
 				    maildir_len) == 0 &&
-			    (p[maildir_len-1] == '\0' ||
-			     p[maildir_len-1] == '/')) {
+			    (p[maildir_len] == '\0' ||
+			     p[maildir_len] == '/')) {
 				/* don't allow maildir_name to be used as part
 				   of the mailbox name */
 				return FALSE;
