@@ -1,5 +1,5 @@
-#ifndef __IMAP_QUOTA_PLUGIN_H
-#define __IMAP_QUOTA_PLUGIN_H
+#ifndef IMAP_QUOTA_PLUGIN_H
+#define IMAP_QUOTA_PLUGIN_H
 
 void imap_quota_plugin_init(void);
 void imap_quota_plugin_deinit(void);
